"""This module helps convert to varios data unit"""


def to_kb(d: int | float, un:str)-> float:
    """Convert from MB, GB or TB to KB. 
    d:  data to convert, can be of type int or float.
    un: unit name, of type string"""

    kb = 0
    unit = "KB"
    unitNames = ["M", "G", "T"]
    
    un = un.upper()

    if un in unitNames:
        if un == "M":
            kb = d * 1_000
            return f"{kb:,.2f} {unit}"

        elif un == "G":
            kb = d * 1_000_000
            return f"{kb:,.2f} {unit}"

        elif un == "T":
            kb = d * 1_000_000_000
            return f"{kb:,.2f} {unit}"
    
    else:
        print(f"{un} is not a valid data unit name. Please try again by entering any of {unitNames}")
        return 0




def to_mb(d: int | float, un:str)-> float:
    """Convert from KB, GB, or TB to MB 
    d:  data to convert, can be of type int or float.
    un: unit name, of type string"""

    mb = 0
    unit = "MB"
    unitNames = ["K", "G", "T"]

    un = un.upper()

    if un in unitNames:
        if un == "K":
            mb = d / 1_000
            return f"{mb:,.2f} {unit}"

        elif un == "G":
            mb = d * 1_000
            return f"{mb:,.2f} {unit}"

        elif un == "T":
            mb = d * 1_000_000
            return f"{mb:,.2f} {unit}"
    
    else:
        print(f"{un} is not a valid data unit name. Please try again by entering any of {unitNames}")
        return 0


def to_gb(d: int | float, un:str)-> float:
    """Convert from KB, MB, or TB to GB 
    d:  data to convert, can be of type int or float.
    un: unit name, of type string"""

    gb = 0
    unit = "GB"
    unitNames = ["K", "M", "T"]

    un = un.upper()

    if un in unitNames:
        if un == "K":
            gb = d / 1_000_000
            return f"{gb:,.2f} {unit}"

        elif un == "M":
            gb = d / 1_000
            return f"{gb:,.2f} {unit}"

        elif un == "T":
            gb = d * 1_000
            return f"{gb:,.2f} {unit}"

    else:
        print(f"{un} is not a valid data unit name. Please try again by entering any of {unitNames}")
        return 0


def to_tb(d: int | float, un:str)-> float:
    """Convert from KB, MB. or GB to TB 
    d:  data to convert, can be of type int or float.
    un: unit name, of type string"""

    tb = 0
    unit = "TB"
    unitNames = ["K", "M", "G"]

    un = un.upper()

    if un in unitNames:
        if un == "K":
            tb = d / 1_000_000_000
            return f"{tb:,.2f} {unit}"

        elif un == "M":
            tb = d / 1_000_000
            return f"{tb:,.2f} {unit}"

        elif un == "G":
            tb = d / 1_000
            return f"{tb:,.2f} {unit}"

    else:
        print(f"{un} is not a valid data unit name. Please try again by entering any of {unitNames}")
        return 0


"""A Module that helps Convert Bytes to KB, MB, GB, or TB."""

def bytes_converter(b: int|float) -> float:
    """Converts bytes into KB, MB, GB, or TB"""

    unitSize = 1_000
    unitNames = ["bytes", "KB", "MB", "GB", "TB"]

    for un in unitNames:
        if b < unitSize:
            return f"{b:,.3f} {un}"
        b /= unitSize
        
def data_to_bytes(d:int|float, un: str)-> float:
    """Converts from any of KB, MB, GB, or TB to bytes"""
    unitNames= ["K","M","G","T"]
    b = 0

    unit = un.upper()

    if unit in unitNames:
        if unit == "K":
            b = d*1_000
            return f"{b:,.3f} bytes"

        elif unit == "M":
            b = d*1_000_000
            return f"{b:,.3f} bytes"

        elif unit == "G":
            b = d*1_000_000_000
            return f"{b:,.3f} bytes"

        elif unit == "T":
            b = d*1_000_000_000_000
            return f"{b:,.3f} bytes"
    else:
        print(f"{un} is not a valid data unit name. Please try again by entering any of {unitNames}")
        return 0


import sys
from getopt import getopt, GetoptError

from duconv import bytesconvert
from duconv import convert
from duconv import __version__ as _v

HELPER = f"run the command with the -h or --help option to recieve help"


def help() -> str:
    """Prints usage for when running as a cli app."""

    usage = f"""HELP:\nDESCRIPTION\nThis is a package that can be run from the cli to help in converting digital data units.
COMMANDS
Usage:duconv [-bdgkmthv] [--byte] [--data-to-byte] [--kb] [--mb] [--gb] [--tb] [--help] [--version]
-b, --byte\t\tConvert bytes to KB, MB, GB, and TB
-d, --data-tobyte\tConvert KB, MB, GB, and TB to bytes
-k, --kb\t\tConvert MB, GB, and TB to KB
-m, --mb\t\tConvert KB, GB, and TB to MB
-g, --gb\t\tConvert KB, MB, and TB to GB
-t, --tb\t\tConvert KB, MB, and GB to TB
-v, --version\t\tShow the version
-h, --help\t\tShow usage.
    
e.g: to convert 5 MB in to KB you ust run:
     
     duconv [-k | --kb] 5 M

usable e.g:1
    
     duconv -k 5 M

to convert bytes to KB, MB, GB, TB run:
    
     duconv -b 5000

for other conversion see usable e.g:1
"""

    return usage


def cli() -> None:
    cliArg = sys.argv[1:]
    shortOpt = "b:d:k:m:g:t:vh"
    longOption = [
        "byte=",
        "data-to-byte=",
        "kb=",
        "mb=",
        "gb=",
        "tb=",
        "help",
        "version",
    ]

    try:
        opts, args = getopt(cliArg, shortOpt, longOption)

    except GetoptError as error:
        print(f"{error}\n{HELPER}")

    else:
        if len(opts) < 1:
            print("no options was given")
            sys.exit(f"{HELPER}\nexit code=1")

        for opt, val in opts:
            try:
                if opt in ("-b", "--byte"):
                    print(bytesconvert.bytes_converter(float(val)))

                elif opt in ("-d", "--data-to-byte"):
                    print(bytesconvert.data_to_bytes(float(val), args[0]))

                elif opt in ("-k", "--kb"):
                    print(convert.to_kb(float(val), args[0]))

                elif opt in ("-m", "--mb"):
                    print(convert.to_mb(float(val), args[0]))

                elif opt in ("-g", "--gb"):
                    print(convert.to_gb(float(val), args[0]))

                elif opt in ("-t", "--tb"):
                    print(convert.to_tb(float(val), args[0]))

                elif opt in ("-h", "--help"):
                    print(help())

                elif opt in ("-v", "--version"):
                    print(
                        f"duconv version {_v}\n\nCopyright 2023 Shall Mcfield.\nLicense under the MIT License"
                    )

            except (ValueError, Exception) as e:
                sys.exit(f"{HELPER}\nnumeric value required (exit code=1)")


if __name__ == "__main__":
    cli()



import unittest
import pytest
from duconv import bytesconvert


def test_bytes_converter() -> None:
    assert bytesconvert.bytes_converter(5_482) == "5.482 KB"

    assert bytesconvert.bytes_converter(5_482_000) == "5.482 MB"

    assert bytesconvert.bytes_converter(5_482_100_000) == "5.482 GB"

    assert bytesconvert.bytes_converter(5_482_200_500_000) == "5.482 TB"


def test_data_to_bytes() -> None:
    assert bytesconvert.data_to_bytes(5.482, "k") == "5,482.000 bytes"

    assert bytesconvert.data_to_bytes(5.482, "M") == "5,482,000.000 bytes"

    assert bytesconvert.data_to_bytes(5.482, "g") == "5,482,000,000.000 bytes"

    assert bytesconvert.data_to_bytes(5.482, "t") == "5,482,000,000,000.000 bytes"

    assert bytesconvert.data_to_bytes(5.482, "b") == 0



import unittest
import pytest
from duconv import convert

# class Testconverter(unittest.TestCase): # Uncomment for unittest module


def test_to_kb() -> None:  # Add self if using unittest module.
    # From MB to KB
    kb = convert.to_kb(5, "m")
    assert (
        kb == "5,000.00 KB"
    ), "1 MB = 1,000 KB - resoult should be 5,000.00 KB (5*1,000=5,000.00)"

    # Form GB to KB
    kb = convert.to_kb(5, "G")
    assert (
        kb == "5,000,000.00 KB"
    ), "1 GB = 1,000,000 KB(1,000^2) - resoult should be 5,000,000.00 KB (5*(1,000^2)=5,000,000.00)"

    # Form TB to KB
    kb = convert.to_kb(5, "t")
    assert (
        kb == "5,000,000,000.00 KB"
    ), "1 TB = 1,000,000,000 KB(1000^3) - resoult should be 5,000,000,000.00 KB (5*(1,000^3)=5,000,000,000.00)"

    # Invalid unit name check
    kb = convert.to_kb(5, "k")
    assert (
        kb == 0
    ), "Return value is 0 when unit name is invalid - Unit name for to_kb fn must be one of M,G,T"


def test_to_mb() -> None:  # Add self if using unittest module.
    # From KB to MB
    mb = convert.to_mb(2_500, "K")
    assert (
        mb == "2.50 MB"
    ), "1,000 KB = 1 MB - resoult should be 2.50 MB (2,500/1,000=2.50)"

    # From GB to MB
    mb = convert.to_mb(2.5, "g")
    assert (
        mb == "2,500.00 MB"
    ), "1 GB = 1,000 MB - resoult should be 2,500.00 MB (2.5*1,000=2,500)"

    # From TB to MB
    mb = convert.to_mb(1.92, "t")
    assert (
        mb == "1,920,000.00 MB"
    ), "1 TB = 1,000,000 MB (1,000^2) - resoult should be 1,920,000.00 MB (1.92*(1,000^2)=1,920,000.00)"

    # Invalid unit name check
    mb = convert.to_mb(5, "M")
    assert (
        mb == 0
    ), "Return value is 0 when unit name is invalid - Unit name for to_mb fn must be one of K,G,T"


def test_to_gb() -> None:  # Add self if using unittest module.
    # From KB to GB
    gb = convert.to_gb(9_207_000, "k")
    assert (
        gb == "9.21 GB"
    ), "1 GB = 1,000,000 KB - resoult should be 9.20 GB (9,207,000/1,000,000=9.20)"

    # From MB to GB
    gb = convert.to_gb(6_270, "M")
    assert (
        gb == "6.27 GB"
    ), "1 GB = 1,000 MB - resoult should be 6.27 GB (6,270/1,000=6.27)"

    # From TB to GB
    gb = convert.to_gb(0.23, "t")
    assert (
        gb == "230.00 GB"
    ), "1 TB = 1,000 GB - resoult should be 230.00 GB (0.23*1,000=230)"

    # Invalid unit name check
    gb = convert.to_gb(5, "g")
    assert (
        gb == 0
    ), "Return value is 0 when unit name is invalid - Unit name for to_gb fn must be one of K,M,T"


def test_to_tb() -> None:  # Add self if using unittest module.
    # From KB to TB
    tb = convert.to_tb(7_981_905_000, "K")
    assert (
        tb == "7.98 TB"
    ), "1 TB =  1,000,000,000 KB (1,000^3) - resoult should be 7.98 TB (7,981,905,000/(1,000^3)=7.98)"

    # From MB to TB
    tb = convert.to_tb(5_971_000, "m")
    assert (
        tb == "5.97 TB"
    ), "1 TB =  1,000,000 MB (1,000^2) - resoult should be 5.97 TB (5,971,000/(1,000^2)=5.97)"

    # From GB to TB
    tb = convert.to_tb(9_100, "g")
    assert (
        tb == "9.10 TB"
    ), "1 TB =  1,000 GB - resoult should be 9.10 TB (9,100/1,000=9.10)"

    # Invalid unit name check
    tb = convert.to_tb(5, "T")
    assert (
        tb == 0
    ), "Return value is 0 when unit name is invalid - Unit name for to_tb fn must be one of K,M,G"


# Use this line when using unittest and want to exce from the script it self.
# if __name__ == "__main__":
#     unittest.main()
